# Electing Local Election Officials Replication Code: Data Prep
## This code creates the main dataset used to make most figures and tables, and saves it to "dat.csv"

# Install necessary packages and versions 
## NOTE: This will install the exact versions of the packages used. If you do not want your current versions
## overwritten, then do not run lines 8-43 and instead install these packages manually.

# Install devtools if not already installed
if (!requireNamespace("devtools", quietly = TRUE)) install.packages("devtools")
library(devtools)

# List of required packages and versions
packages <- list(
  "DIDmultiplegt" = "2.0.0",
  "data.table" = "1.16.0",
  "did" = "2.1.2",
  "fixest" = "0.12.1",
  "gsynth" = "1.2.1",
  "haven" = "2.5.4",
  "kableExtra" = "1.4.0",
  "mapproj" = "1.2.11",
  "maps" = "3.4.2",
  "PanelMatch" = "2.2.0",
  "priceR" = "1.0.2",
  "RColorBrewer" = "1.1-3",
  "survey" = "4.4-2",
  "tidyverse" = "2.0.0",
  "tidycensus" = "1.6.5",
  "usdata" = "0.3.1"
)

# Install missing or outdated packages
for (pkg in names(packages)) {
  required_version <- packages[[pkg]]
  
  if (!pkg %in% rownames(installed.packages()) ||
      packageVersion(pkg) != required_version) {
    
    message(sprintf("Installing %s version %s", pkg, required_version))
    install_version(pkg, version = required_version, repos = "http://cran.us.r-project.org")
  } else {
    message(sprintf("%s version %s already installed", pkg, required_version))
  }
}


library(tidyverse); library(tidycensus); library(priceR)

# Combine election administration datasets
state_names <- c(
  "az" = "Arizona", "ca" = "California", "ga" = "Georgia", "il" = "Illinois",
  "in" = "Indiana", "mn" = "Minnesota", "mo" = "Missouri", "mt" = "Montana",
  "ne" = "Nebraska", "nv" = "Nevada", "or" = "Oregon", "tx" = "Texas", "wa" = "Washington")
states <- names(state_names)
dat <- map_dfr(states, function(state) {
  file_path <- paste0("../raw_data/admin_data/", state, "_election_admin.csv")
  df <- read_csv(file_path) %>% mutate(state = state_names[[state]])
  return(df)
})

dat <- dat %>% 
  mutate(county = str_to_title(county),
         office_leo = str_to_title(office_leo),
         appointed = case_when(
           elected == 0 ~ 1,
           elected == 1 ~ 0,
           TRUE ~ NA),
         divided_jurisdiction = ifelse(divided_jurisdiction %in% c(NA, 0), 0, 1),
         elected_foia = ifelse(state != "Texas", elected, elected_foia),
         partisan_admin_foia = ifelse(state != "Texas", partisan_admin, partisan_admin_foia),
         elected_noimp = ifelse(state != "Texas", elected, elected_noimp),
         partisan_admin_noimp = ifelse(state != "Texas", partisan_admin, partisan_admin_noimp)) %>% 
  select(state, county, year, office_leo, appointed, partisan_admin, everything())

## Attach FIPS codes
fips <- fips_codes %>% 
  mutate(fips = paste0(state_code, county_code)) %>% 
  rename(state_abr = "state",
         state = "state_name") %>% 
  mutate(county = gsub("County", "", as.character(county)),
         county = str_to_title(county)) %>%
  select(state, state_abr, county, fips)
fips$county <- str_trim(fips$county)

dat <- left_join(dat, fips, by= c("state", "county")) %>% 
  mutate(fips = as.numeric(fips))

# Add election official turnover and EAVS data
dat <- left_join( dat, read.csv("../raw_data/turnover_eavs_dat.csv"), by = c("fips", "year"))

# Add turnout, registration, and partisan vote share data
dat <- left_join(dat, read.csv("../raw_data/turnout_reg_vs.csv"), by =c("fips", "year"))

# Add election cost data and adjust for inflation
cost_dat <- read.csv("../raw_data/elect_expnd.csv") %>% 
  mutate(real_expnd = adjust_for_inflation(price = elect_tot_expnd,
                                           from_date= year,
                                           country = "US",
                                           to_date = 2020,
                                           inflation_dataframe = retrieve_inflation_data("US"),
                                           countries_dataframe = show_countries()),
         fips = as.numeric(fips)) %>% 
  select(fips, year, elect_tot_expnd, real_expnd)

dat <- left_join(dat, cost_dat, by =c("fips", "year")) %>% 
  mutate(reg = vap*reg_pc,
         log_expnd = log(real_expnd),
         log_expend_lead = lead(log_expnd),
         expnd_per_reg = real_expnd/reg,
         expnd_per_reg_lead = lead(expnd_per_reg),
         expnd_per_vap = real_expnd/vap,
         expnd_per_vap_lead = lead(expnd_per_vap),
         log_expnd_per_reg = log(expnd_per_reg),
         log_expnd_per_reg_lead = lead(log_expnd_per_reg),
         log_expnd_per_vap = log(expnd_per_vap),
         log_expnd_per_vap_lead = lead(log_expnd_per_vap))

# Add media data 
dat <- left_join(dat, read.csv("../raw_data/newspaper_dat.csv"), by =c("fips", "year"))

# Add LEO communication data
dat <- left_join(dat, read.csv("../raw_data/comm_dat.csv"), by =c("fips", "year"))

# Calculate interacted fixed effects quartiles
## Calculate pop_by_year FEs
pop <- read.csv("../raw_data/nhgis0007_ts_nominal_county.csv") %>% 
  pivot_longer(cols = starts_with("A00AA"),
               names_to = "year",
               names_prefix = "A00AA",
               values_to = "pop") %>% 
  filter(year == 1960) %>% 
  rename(state = "STATE",
         county = "COUNTY") %>% 
  drop_na() %>%
  filter(state != "District Of Columbia") %>% 
  select(state, county, year, pop) %>% 
  mutate(county = str_remove_all(county, "County"),
         county = str_trim(county)) %>% 
  group_by(state) %>% 
  mutate(pop_quart = cut(pop, quantile(pop, c(0, 1/4, 1/2, 3/4, 1)), labels=c(1, 2, 3, 4))) %>%
  mutate(pop_quart = ifelse(is.na(pop_quart) == TRUE, 1, pop_quart)) %>%
  ungroup() %>% 
  mutate(county = str_to_title(county)) %>% 
  left_join(fips, by = c("state", "county")) %>% 
  mutate(fips = as.numeric(fips)) %>% 
  select(fips, pop_quart)

## calculate dem_vs-by-year FEs
dem_vs_q <- dat %>% 
  filter(year==1968) %>%
  select(state, fips, dem_vs_President) %>%
  drop_na(dem_vs_President) %>% 
  group_by(state) %>% 
  mutate(dem_vs_quart = cut(dem_vs_President, quantile(dem_vs_President, c(0, 1/4, 1/2, 3/4, 1)),
                            labels=c(1, 2, 3, 4))) %>%
  mutate(dem_vs_quart = ifelse(is.na(dem_vs_quart) == TRUE, 1, dem_vs_quart)) %>%
  ungroup() %>% 
  select(fips, dem_vs_quart)

dat <- left_join(dat, pop) %>% 
  left_join(dem_vs_q) %>% 
  mutate(year_demvs = year*dem_vs_quart,
         year_pop = year*pop_quart)

# Add Fixed effects, lead/lag, cohort, and other helpful columns
dat <- dat %>% 
  mutate(cycle = ifelse(year %% 4 == 0, "pres", "midterm"),
         statecounty = paste0(state, county),
         year_state = paste0(state, year),
         year_state_demvs = paste0(state, year_demvs),
         year_state_pop = paste0(state, year_pop),
         small = ifelse(pop_quart <=2, 1, 0),
         democratic = ifelse(dem_vs_quart <= 2, 0, 1),
         year_state_small = paste0(year_state, small),
         year_state_demvs_small = paste0(year_state_demvs, small),
         fips_small = paste0(fips, "_", small),
         appointedXsmall = appointed*small,
         fips_dem = paste0(fips, "_", democratic),
         year_state_dem = paste0(year_state, democratic),
         year_state_pop_dem = paste0(year_state_pop, democratic),
         appointedxdem = appointed*democratic,
         year_state_news = paste0(year_state, any_dailies_constant),
         year_state_news_small = paste0(year_state_news, small),
         year_state_demvs_news = paste0(year_state_demvs, any_dailies_constant),
         year_state_pop_news = paste0(year_state_pop, any_dailies_constant),
         fips_news = paste0(fips, any_dailies_constant),
         appointedXnews = appointed*any_dailies_constant,
         year_state_news_alt = paste0(year_state, any_dailies),
         year_state_demvs_news_alt = paste0(year_state_demvs, any_dailies),
         year_state_pop_news_alt = paste0(year_state_pop, any_dailies),
         fips_news_alt = paste0(fips, any_dailies),
         appointedXnews_alt = appointed*any_dailies,
         appointed_noimp = ifelse(elected_noimp == 1, 0, 1),
         appointed_foia = ifelse(elected_foia == 1, 0, 1),
         probate_judge = ifelse(is.na(office_leo), 0, ifelse(office_leo == "Probate Judge", 1, 0)),
         clerk = ifelse(is.na(office_leo), 0, ifelse(office_leo %in% c("County Clerk", "District & County Clerk",
                                          "Clerk and Recorder", "Recorder"), 1, 0)),
         tax_assessor = ifelse(is.na(office_leo), 0, ifelse(office_leo == "Tax Assessor-Collector", 1, 0)),
         auditor = ifelse(is.na(office_leo), 0, ifelse(office_leo %in% c("Auditor", "County Auditor", "County Assessor"), 1, 0)),
         no_preclearance = ifelse(year >= 2014, 1, 0),
         reg_separate = ifelse(state %in% c("Arizona", "Georgia"), 1, 0),
         sole_authority = ifelse(state %in% c("California", "Illinois", "Missouri", "Montana",
                                              "Nebraska", "Nevada", "Oregon", "Washington"), 1, 0)) %>% 
  group_by(fips) %>% 
  arrange(fips, year) %>% 
  mutate(ever_divided = max(divided_jurisdiction),
         evertreated = max(appointed),
         appointedlead1 = lead(appointed),
         appointedlead2 = lead(appointedlead1),
         appointedlead3 = lead(appointedlead2),
         appointedlead4 = lead(appointedlead3),
         electedlag1 = lag(elected),
         appointedlag1 = lag(appointed),
         appointedlag2 = lag(appointedlag1),
         appointedlag3 = lag(appointedlag2),
         appointedlag4 = lag(appointedlag3),
         appointedleaddif1 = ifelse(appointed == 0 & appointedlead1 == 1, 1,
                                    ifelse(is.na(appointed) | is.na(appointedlead1), NA, 0)),
         appointedleaddif2 = ifelse(appointed == 0 & appointedlead1 == 0 & appointedlead2 == 1, 1,
                                    ifelse(is.na(appointed) | is.na(appointedlead2), NA, 0)),
         appointedleaddif3 = ifelse(appointed == 0 & appointedlead2 == 0 & appointedlead3 == 1, 1,
                                    ifelse(is.na(appointed) | is.na(appointedlead3), NA, 0)),
         appointedleaddif4 = ifelse(appointed == 0 & appointedlead3 ==0 & appointedlead4 == 1,  1,
                                    ifelse(is.na(appointed) | is.na(appointedlead4), NA, 0)),
         temp = abs(0 - appointed - appointedlead1),
         elected0 = ifelse(appointed == 0 & appointedlag1 == 1, 1, 
                           ifelse(is.na(appointed) | is.na(appointedlag1), NA, 0)),
         appointed0 = ifelse(appointed == 1 & appointedlag1 == 0, 1,
                             ifelse(is.na(appointed) | is.na(appointedlag1), NA, 0)),
         appointedlagdif1 =  ifelse(appointed == 1 & appointedlag1 == 1 & appointedlag2 ==0, 1,
                                    ifelse(is.na(appointed) | is.na(appointedlag1), NA, 0)),
         appointedlagdif2 = ifelse(appointed ==1 & appointedlag2 == 1 & appointedlag3 ==0, 1,
                                   ifelse(is.na(appointed) | is.na(appointedlag2), NA, 0)),
         appointedlast =  ifelse(appointed == 1 & appointedlag3 == 1, 1,
                                 ifelse(is.na(appointed) | is.na(appointedlag3), NA, 0)),
         appoint_to_elect = ifelse(ever_divided == 1, NA, abs(min(appointed-appointedlag1, na.rm=TRUE))),
         elect_to_appoint = ifelse(ever_divided == 1, NA, abs(max(appointed-appointedlag1, na.rm = TRUE))),
         switcher = abs(max(appointed)-min(appointed)),
         multiple_switches = ifelse(appoint_to_elect == 1 & elect_to_appoint == 1, 1, 0),
         min_appoint = min(appointed),
         max_appoint = max(appointed),
         always_appointed = ifelse(min_appoint==1 & max_appoint==1, 1, 0),
         always_elected = ifelse(min_appoint==0 & max_appoint == 0, 1, 0)) %>% 
  ungroup() %>% 
  mutate(initiator = case_when(
    state %in% c("Georgia", "Missouri") & switcher == 1 ~ "state_leg",
    state == "Nebraska" & county == "Sarpy" & switcher == 1 ~ "state_leg",
    state %in% c("Arizona", "Illinois", "Nevada", "Texas") & switcher == 1 ~ "county_leg",
    state == "California" & county %in% c("Alameda", "Orange", "Sacramento", "El Dorado",
                                          "Imperial", "Kings", "Lake",
                                          "Merced", "Monterey", "Riverside", "San Joacquin",
                                          "Solano", "Tulare") & switcher == 1 ~ "county_leg",
    state == "Montana" & county %in% c("Big Horn", "Carbon", "Lake", "Lincoln", "Missoula",
                                       "Sanders", "Yellowstone") & switcher == 1 ~ "county_leg",
    state == "Nebraska" & county %in% c("Buffalo", "Cass", "Hall", "Platte") & switcher == 1 ~ "county_leg",
    state %in% c("Oregon", "Washington") & switcher == 1 ~ "county_ref",
    state == "California" & county %in% c("Mono", "Marin") & switcher == 1 ~ "county_ref",
    state == "Montana" & county == "Deer Lodge" & switcher == 1 ~ "county_ref",
    TRUE ~ "non_switcher"),
    init_state_leg = ifelse(initiator == "state_leg", 1, 0),
    init_county_leg = ifelse(initiator == "county_leg", 1, 0),
    init_county_ref = ifelse(initiator == "county_ref", 1, 0),
    package_reform_binary = case_when(
      state %in% c("Oregon") & switcher == 1 ~ 1,
      state == "California" & county == "Marin" & switcher == 1 ~ 1,
      TRUE ~ 0)) %>% 
  ungroup()

temp <- dat %>%
  mutate(year = as.numeric(year)) %>% 
  group_by(fips) %>% 
  arrange(fips, year) %>% 
  mutate(cohort = if_else(appointed == 1, year, max(year)+2)) %>% 
  slice(which.min(cohort)) %>% 
  select (fips, cohort) %>% 
  mutate(cohort = if_else(cohort ==2024, 0, cohort))

dat <- left_join(dat, temp, by = "fips") %>% 
  select(1:8, cohort, evertreated, everything())

# Save the dataset
write.csv(dat, "../dat.csv", row.names = F)